import java.util.Vector;
import java.awt.*;

/**
 * BiPartiter Graph mit 2 Knotenmengen und einer Kantenmenge.
 * Enthlt bereits Methoden und Attribute zur grafischen Darstellung.
 */
public class BiGraph
{
    // 2 Knotenmengen
    private Vector aVertices, bVertices;
    // Kanten
    private Vector edges;
    
    private int aCounter, bCounter, eCounter;
    private int aIsTagged, bIsTagged;

    private final static Color aColors[]={Color.RED, Color.MAGENTA, 
					  Color.ORANGE};
    private final static Color bColors[]={Color.BLUE, Color.CYAN, 
					  Color.ORANGE};

    /**
     * Erzeugt einen leeren Graphen.
     */
    BiGraph()
    {
	aCounter=0;
	bCounter=0;
	aVertices=new Vector();
	bVertices=new Vector();
	edges=new Vector();
	aIsTagged=-1;
	bIsTagged=-1;
    }

    /**
     * Fgt einen Sink-Knoten hinzu.
     * @param x X-Koordinate des Knotens
     * @param y Y-Koordinate des Knotens
     * @param d Durchmesser des Knotens
     */
    public void addAVertex(int x, int y, int d)
    {
	Vertex toAdd=new Vertex(aVertices.size(),x,y,d,aColors);
	aVertices.addElement(toAdd);
    }
 
    /**
     * Fgt einen Source-Knoten hinzu.
     * @param x X-Koordinate des Knotens
     * @param y Y-Koordinate des Knotens
     * @param d Durchmesser des Knotens
     */
    public void addBVertex(int x, int y, int d)
    {
	Vertex toAdd=new Vertex(bVertices.size(),x,y,d,bColors);
	bVertices.addElement(toAdd);
    }

    /**
     * Setzt alle laufenden Abfragen zurck.
     */
    public void rewindAll()
    {
	aCounter=0;
	bCounter=0;
	eCounter=0;
    }
    
    /**
     * Liefert den nchsten Sink Knoten oder null, wenn es keinen gibt.
     * @return nchster Sink Knoten
     */
    public Vertex getNextAVertex()
    {
	if(aCounter<aVertices.size())
	{
	    aCounter++;
	    return((Vertex)aVertices.elementAt(aCounter-1));
	}
	return(null);
    }
    
    /**
     * Liefert den nchsten Source Knoten oder null, wenn es keinen gibt.
     * @return nchster Source Knoten
     */
    public Vertex getNextBVertex()
    {
	if(bCounter<bVertices.size())
	{
	    bCounter++;
	    return((Vertex)bVertices.elementAt(bCounter-1));
	}
	return(null);
    }

    /**
     * Liefert die nchste Kante oder null, wenn es keine gibt.
     * @return nchste Kante
     */
    public Edge getNextEdge()
    {
	if(eCounter<edges.size())
	{
	    eCounter++;
	    return((Edge)edges.elementAt(eCounter-1));
	}
	return(null);
    }

    /**
     * Liefert den Source Knoten mit entsprechender Nummer.
     * @param vnr Nummer des Knotens
     * @return Knoten mit der Nummer vnr
     */
    public Vertex getVertexBByNumber(int vnr)
    {
	return((Vertex)bVertices.elementAt(vnr));
    }
 
    /**
     * Liefert den Sink Knoten mit entsprechender Nummer.
     * @param vnr Nummer des Knotens
     * @return Knoten mit der Nummer vnr
     */
    public Vertex getVertexAByNumber(int vnr)
    {
	return((Vertex)aVertices.elementAt(vnr));
    }

    /**
     * Ermittelt die Anzahl der Sink Knoten.
     * @return Anzahl der Sink Knoten
     */
    public int getACount()
    {
	return(aVertices.size());
    }
    
    /**
     * Ermittelt die Anzahl der Source Knoten.
     * @return Anzahl der Source Knoten
     */
    public int getBCount()
    {
	return(bVertices.size());
    }

    /**
     * @return aktuelle Sink-Knoten-Nr
     */
    public int getCurrentA()
    {
	return(aCounter-1);
    }

    /**
     * @return aktuelle Source-Knoten-Nr
     */
    public int getCurrentB()
    {
	return(bCounter-1);
    }

    /**
     * (De)Markiert Knoten.
     * @param x X-Koordinate des Zielpunktes
     * @param y Y-Koordinate des Zielpinktes
     * @return false, wenn nichts verndert wurde, true sonst
     */
    public boolean tagForEdgeEdit(int x, int y)
    {
	int i;
	for(i=0;i<aVertices.size();i++)
	{
	    //ein Sink Knoten bei x,y wurde gefunden
	    if(getVertexAByNumber(i).thatMeansMe(x,y))
	    {
		//Sonderfall, der angeclickte Knoten ist bereits markiert
		//oder markiert und wieder demarkiert worden
		if(i==aIsTagged)
		{
		    //Markierung tooglen
		    getVertexAByNumber(i).setEdgeEdit();
		    if(!getVertexAByNumber(i).edgeEdit) aIsTagged=-1;
		    return(true);
		}
		//ist bereits ein Sink-Knoten markiert, demarkiere diesen
		if(aIsTagged>=0) getVertexAByNumber(aIsTagged).setEdgeEdit();
		//markiere den angeclickten Sink Knoten
		getVertexAByNumber(i).setEdgeEdit();
		aIsTagged=i;
		return(true);
	    }
	}
	for(i=0;i<bVertices.size();i++)
	{
	    //ein Source Konten bei x,y wurde gefunden
	    if(getVertexBByNumber(i).thatMeansMe(x,y))
	    {
		//Sonderfall, der angeclickte Knoten ist bereits markiert
		//oder markiert und wieder demarkiert worden
		if(i==bIsTagged)
		{
		    //Markierung tooglen
		    getVertexBByNumber(i).setEdgeEdit();
		    if(!getVertexBByNumber(i).edgeEdit) bIsTagged=-1;
		    return(true);
		}
		//ist bereits ein Source-Knoten markiert, demarkiere diesen
		if(bIsTagged>=0) getVertexBByNumber(bIsTagged).setEdgeEdit();
		//markiere den angeclickten Source Knoten
		getVertexBByNumber(i).setEdgeEdit();
		bIsTagged=i;
		return(true);
	    }
	}
	return(false);
    }

    /**
     * Erzeugt eine neue Kante, wenn entsprechende Knoten markiert sind.
     */
    public void checkAndInsertEdge()
    {
	if((aIsTagged<0)||(bIsTagged<0)) return;
	Edge aTob=new Edge(getVertexAByNumber(aIsTagged), 
			   getVertexBByNumber(bIsTagged));
	Edge bToa=new Edge(getVertexBByNumber(bIsTagged),
			   getVertexAByNumber(aIsTagged));
	getVertexAByNumber(aIsTagged).addEdge(aTob);
	getVertexBByNumber(bIsTagged).addEdge(bToa);
	edges.addElement(aTob);
	getVertexAByNumber(aIsTagged).setEdgeEdit();
	getVertexBByNumber(bIsTagged).setEdgeEdit();
	aIsTagged=-1;
	bIsTagged=-1;
    }

    /**
     * Erzeugt eine Kante zwischen 2 Knoten.
     * @param a Nummer des Sink-Knotens
     * @param b Nummer des Source-Knotens
     */
    public void addEdgeByNr(int a, int b)
    {
	if(aIsTagged>=0) getVertexAByNumber(aIsTagged).setEdgeEdit();
	if(bIsTagged>=0) getVertexBByNumber(bIsTagged).setEdgeEdit();
	getVertexAByNumber(a).setEdgeEdit();
	getVertexBByNumber(b).setEdgeEdit();
	aIsTagged=a;
	bIsTagged=b;
	checkAndInsertEdge();
    }

    /**
     * Setzt die Farbe von allen Sink-Knoten auf den Initialwert zurck.
     */
    public void resetAllAColor()
    {
	int i;
	for(i=0;i<aVertices.size();i++)
	    getVertexAByNumber(i).setMark(0);
    }

    /**
     * Setzt die Farbe von allen Source-Knoten auf den Initialwert zurck.
     */
    public void resetAllBColor()
    {
	int i;
	for(i=0;i<bVertices.size();i++)
	    getVertexBByNumber(i).setMark(0);
    }
    
    /**
     * Setzt die Farbe von allen Kanten auf den Initialwert zurck.
     */
    public void resetAllEdgeColor()
    {
 	int i;
	for(i=0;i<edges.size();i++)
	{
	    ((Edge)edges.elementAt(i)).setColorNr(0);
	}
    }

    /**
     * Setzt die Dicke von allen Kanten auf den Initialwert zurck.
     */
    public void resetAllEdgeThickness()
    {
	int i;
	for(i=0;i<edges.size();i++)
	{
	    ((Edge)edges.elementAt(i)).setThickness(1);
	}
    }
    
    /**
     * Setzt alle Attribute von allen Kanten auf den Initialwert zurck.
     */
    public void resetAllEdgeAttr()
    {
	resetAllEdgeColor();
	resetAllEdgeThickness();
    }

    /**
     * Setzt alle Attribute von allen Kanten und allen Knoten 
     * auf den Initialwert zurck.
     */
    public void resetAll()
    {
	resetAllAColor();
	resetAllBColor();
	resetAllEdgeAttr();
    }

}




