import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.net.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import javax.swing.text.*;

/**
 * Visualisiert einen Matchingalgorithmus auf Bipartiten Graphen.<br><br>
 * Top-Down-Struktur der verwenden Swing-Komponenten:
 * <pre>

     Container pane
     |
     |
     +-- JPanel globalPanel
	 |
	 |-- JSplitPane subGlobalPanelLeft
	 |   |
	 |   |-- GraphPanel graphPanel
	 |   |
	 |   +-- AlgoPanel algoPanel
	 |
	 +-- JPanel dataPanel
	     |
	     |-- JTabbedPane controlTabbedPane
	     |   |
	     |   |-- JPanel inputPanel
	     |   |
	     |   +-- JPanel algoControlPanel
	     |       |
	     |       |-- JPanel aCNorthPanel
	     |       |
	     |       +-- JPanel aCSouthPanel
	     |
	     +-- JScrollPane dataScrollPane
	         |
		 + JEditorPane dataArea

 * </pre>
 * @author  Thomas Elste
 * @version 0.3.1
 */
public class BiPartGraph extends JApplet implements MouseListener,ActionListener,ItemListener, ChangeListener
{
    //Swing Components
    private JPanel globalPanel,inputPanel, dataPanel,algoJPanel,
	algoControlPanel,aCSouthPanel, aCNorthPanel;
    private JSplitPane subGlobalLeftPane;
    private JTabbedPane controlTabbedPane;
    private JLabel randLabel1, randLabel2;
    private JEditorPane dataArea;
    private AlgoPanel algoPanel;
    private JTextField vertexAField,vertexBField, randEdgeField;
    private JScrollPane algoScrollPane, dataScrollPane;
    private JSlider srcsSlider, snksSlider;
    private JButton newGraphButton,randomEdgesButton, runAlgButton,
	algStartButton, algStopButton, algStepButton, algFFButton;
   
    //objects from own classes
    private GraphPanel graphPanel;
    private BiGraph graph;
    private MatchAlg1 algo;

    //others
    private Dimension size,gpSize;
    private URL base,imgBase;
    private InetAddress serverAddress;
    
    //constants
    private final Color backgroundColor=new Color(255,255,204);
    private final Color controlColor=new Color(255,200,128);
    private final int maxVertexDiameter=40;
    private final int minVertexDiameter=8;

    /* Versionsnummer */
    public final String version="0.3.1";

    //Konstruktor
    public void init()
    {
	// Klasse fr Matchingalgorithmus instantiieren
	algo=new MatchAlg1(this);
	// gwnschte Gre ermitteln
	size=getSize();
	System.out.println("****** BiPartGraph v"+version+" ******");
	System.out.println("author: Thomas Elste <info@elste.org>");

	try
	{
	    base = getDocumentBase();
	}
	catch (Exception e) {}
	
	try
	{
	    serverAddress = InetAddress.getByName(base.getHost());
	}
	catch (UnknownHostException e) {}
		
	// Pfad zu eventuell bentigten Bilder oder anderen Dateien
	try
	{
	    imgBase=new URL(base.toString().substring(0,base.toString().lastIndexOf("/")+1));
	} catch(MalformedURLException e) {}

	graph=new BiGraph();

	//alle Komponenten initialisieren, 
	//positionieren und die Actionlistener zuweisen
	Container pane = getContentPane();
	
	globalPanel = new JPanel();
	globalPanel.setLayout(new BorderLayout());

	subGlobalLeftPane=new JSplitPane(JSplitPane.VERTICAL_SPLIT);

	graphPanel=new GraphPanel(size.width*2/3, size.height*2/3, graph);
	graphPanel.setPreferredSize(new Dimension(size.width*2/3, 
						  size.height*1/2));
	graphPanel.setMinimumSize(new Dimension(size.width*2/3,
						size.height*1/5));

	algoJPanel=new JPanel(new BorderLayout());
	algoJPanel.setPreferredSize(new Dimension(size.width*2/3,
						 size.height*1/2-14));
	algoJPanel.setMinimumSize(new Dimension(size.width*2/3,
					       size.height*1/3-14));
	algoJPanel.setBackground(backgroundColor);

	URL algoUrl=null;
	try
	{
	    algoUrl=new URL(imgBase.toString()+"match1.cpp");
	} catch (MalformedURLException e) {}


	algoPanel=new AlgoPanel(700,2000);
	//todo: benoetigte zeichenfeldgroesse automatisch ermitteln
	algoPanel.setPreferredSize(new Dimension(700,1400));

	algoScrollPane=new JScrollPane(algoPanel,
		       JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		       JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	algoJPanel.add(algoScrollPane, "Center");

	dataPanel=new JPanel(new BorderLayout());
	dataPanel.setPreferredSize(new Dimension(size.width*1/3,
						 size.height));
	dataPanel.setBackground(backgroundColor);
	dataPanel.setBorder(BorderFactory.createMatteBorder(1,1,2,2,Color.black));

	dataArea=new JEditorPane();
	// Daten sollen als HTML Tabellen angezeigt werden
	dataArea.setContentType("text/html");

	dataScrollPane=new JScrollPane(dataArea,
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	controlTabbedPane=new JTabbedPane();
	controlTabbedPane.setPreferredSize(new Dimension(size.width*1/3-20,
							 size.height*1/3));
	inputPanel=new JPanel(new GridLayout(5,2,3,11));
	inputPanel.setBackground(controlColor);

	snksSlider=new JSlider(1,50,5);
	vertexAField=new JTextField("5", 5);
	snksSlider.setBackground(controlColor);
	snksSlider.setToolTipText("Anzahl der Sink-Knoten");
	vertexAField.setEditable(false);
	srcsSlider=new JSlider(1,50,5);
	srcsSlider.setBackground(controlColor);
	srcsSlider.setToolTipText("Anzahl der Source-Knoten");
	vertexBField=new JTextField("5",5);
	vertexBField.setEditable(false);
	newGraphButton=new JButton("neuer Graph");
	randomEdgesButton=new JButton("% zuf. Kanten");
	randomEdgesButton.setToolTipText("zufllige Kanten erzeugen");
	randLabel1=new JLabel("mit");
	randLabel2=new JLabel("% Wahrscheinlichkeit");
	randEdgeField=new JTextField("50",3);
	randEdgeField.setToolTipText("Wahrscheinlichkeit fr zufllige Kanten (0-100)");
	runAlgButton=new JButton("Matching");

	snksSlider.addChangeListener(this);
	srcsSlider.addChangeListener(this);
	newGraphButton.addActionListener(this);
	randomEdgesButton.addActionListener(this);
	runAlgButton.addActionListener(this);

	inputPanel.add(snksSlider);
	inputPanel.add(vertexAField);

	inputPanel.add(srcsSlider);
	inputPanel.add(vertexBField);

	inputPanel.add(new Label(" "));
	inputPanel.add(newGraphButton);

	inputPanel.add(new Label(" "));
	inputPanel.add(new Label(" "));

	inputPanel.add(randEdgeField);
	inputPanel.add(randomEdgesButton);
	
	algoControlPanel=new JPanel(new BorderLayout());
	algoControlPanel.setBackground(controlColor);
	
	aCNorthPanel=new JPanel();
	aCNorthPanel.setBackground(controlColor);

	aCSouthPanel=new JPanel();
	aCSouthPanel.setBackground(controlColor);
	
	// Icons fr die Buttons laden
	try
	{
	    algStopButton=new JButton(new ImageIcon(
			  new URL(imgBase.toString()+"button_stop.gif")));
	    algStartButton=new JButton(new ImageIcon(
		           new URL(imgBase.toString()+"button_play.gif")));
	    algStepButton=new JButton(new ImageIcon(
		          new URL(imgBase.toString()+"button_step.gif")));
	    algFFButton=new JButton(new ImageIcon(
		        new URL(imgBase.toString()+"button_ff.gif")));
	} 
	catch(MalformedURLException e)
	{
	    System.out.println("error loading button gifs");
	    algStopButton=new JButton("Stop");
	    algStartButton=new JButton("Start");
	    algStepButton=new JButton("Step");
	    algFFButton=new JButton("FF");
	}

	algStopButton.setEnabled(false);
	algStopButton.setToolTipText("anhalten");
	algStepButton.setEnabled(false);
	algStepButton.setToolTipText("ein Schritt");
	algStartButton.setEnabled(false);
	algStartButton.setToolTipText("durchlaufen");
	algFFButton.setEnabled(false);
	algFFButton.setToolTipText("schnell durchlaufen");

	algStopButton.addActionListener(this);
	algStepButton.addActionListener(this);
	algStartButton.addActionListener(this);
	algFFButton.addActionListener(this);

	aCSouthPanel.add(algStopButton);
	aCSouthPanel.add(algStepButton);
	aCSouthPanel.add(algStartButton);
	aCSouthPanel.add(algFFButton);

	algoControlPanel.add(aCNorthPanel, "North");
	algoControlPanel.add(aCSouthPanel, "South");

	controlTabbedPane.addTab("Eingabe", inputPanel);
	controlTabbedPane.addTab("Matching", algoControlPanel);

	dataPanel.add(controlTabbedPane, "North");
	dataPanel.add(dataScrollPane, "Center");

	subGlobalLeftPane.setTopComponent(graphPanel);
	subGlobalLeftPane.setBottomComponent(algoJPanel);

	globalPanel.add(subGlobalLeftPane, "West");
	globalPanel.add(dataPanel, "East");

	pane.add(globalPanel);

	// die start Methode von graphPanel _nach_ dem adden aufgerufen werden
	// erst dann steht die gre fr das BackgroundImage fest
	graphPanel.start();
	graphPanel.repaint();

	graphPanel.addMouseListener(this);
	
	algoPanel.start(imgBase.toString()+"match1.cpp");
	algoPanel.setScrollPane(algoScrollPane);
    }
    
    public void start()
    {
	gpSize=graphPanel.getSize();
	gpSize.height-=4;
	gpSize.width-=4;
    }
 
    /**
     * zeigt eine Warnmeldung an
     *
     * @param wNr Nummer der Warnung
     */
    private void showWarning(int wNr)
    {
	String wMes;
	switch(wNr)
	{
 	   case 0:
	       wMes=new String("No graph edit possible while matching is running.");
	       break;
	   default:
	       wMes=new String("Unknown warning.");
	       break;
	}
	JOptionPane.showMessageDialog(null,
				      "Warning ("+wNr+"): "+wMes,
				      "Warning ...",
				      JOptionPane.WARNING_MESSAGE);
    }

  
    public void mouseClicked(MouseEvent me)
    {
	int mx, my;
	mx=me.getX();
	my=me.getY();
	//mit der rechten Mousetaste kann manuell ein repaint ausgelst werden
	//falls Grafikfehler auftreten
	if(me.getButton()==MouseEvent.BUTTON3)
	{
	    this.repaint();
	    graphPanel.repaint();
	    algoPanel.repaint();
	    return;
	}

	//luft der Algorithmus noch nicht, drfen die Kanten des
	//Graphen editiert werden
	if(!algo.isRunning())
	{
	    if(graph.tagForEdgeEdit(mx,my)) graph.checkAndInsertEdge();
	    graphPanel.repaint();
	    return;
	}
	else
	    showWarning(0);
    }

    public void mouseEntered(MouseEvent me)
    {
    }

    public void mousePressed(MouseEvent me)
    {
    }

    public void mouseReleased(MouseEvent me)
    {
    }

    public void mouseExited(MouseEvent me)
    {
    }

    public void actionPerformed(ActionEvent evt)
    {
	//newGraphButton verwirft den alten Graphen komplett und
	//erzeugt einen neuen
	if(newGraphButton==evt.getSource())
	{
	    int aN,bN;
	    try
	    {
		aN=Integer.parseInt(vertexAField.getText());
		bN=Integer.parseInt(vertexBField.getText());
	    } catch(NumberFormatException e) {return;}
	    createGraph(bN,aN);
	    algStepButton.setEnabled(true);
	}
	//erzeugen von zuflligen Kanten
	else if(randomEdgesButton==evt.getSource())
	{
	    int r;
	    try
	    {
		r=Integer.parseInt(randEdgeField.getText());
	    } catch(NumberFormatException e) {return;}
	    makeRandEdges(r);
	}
	//den Algorithmus starten oder einen Schritt durchfhren
	else if(algStepButton==evt.getSource())
	{
	    if(!algo.isRunning())
	    {
		algStartButton.setEnabled(true);
		algStopButton.setEnabled(true);
		algFFButton.setEnabled(true);
		controlTabbedPane.setEnabledAt(0,false);
		graph.resetAll();
		algo.getMatch(graph,0);
		this.repaint();
	    }
	    else
		algo.doStep();
	}
	//automatischen Durchlauf starten
	else if(algStartButton==evt.getSource())
	{
	    algo.doStart();
	}
	//Algorithmus anhalten
	else if(algStopButton==evt.getSource())
	{
	    algo.doStop();
	}
	//schnellen Durchlaufen ohne Visualisierung
	else if(algFFButton==evt.getSource())
	{
	    algo.doFF();
	}
    }

    public void itemStateChanged(ItemEvent e)	
    {
    }
    
    /**
     * Werte der Slider in den entsprechenden Textfeldern anzeigen
     */
    public void stateChanged(ChangeEvent ce)
    {
	if(ce.getSource()==snksSlider)
	{
	    vertexAField.setText(new Integer(snksSlider.getValue()).toString());
	}
	else if(ce.getSource()==srcsSlider)
	{
	    vertexBField.setText(new Integer(srcsSlider.getValue()).toString());
	}
    }

    /**
     * erzeugt einen neuen Bipartiten Graphen
     * @param aNodes Anzahl der Source-Knoten
     * @param bNodes Anzahl der Sink-Knoten
     */
    public void createGraph(int aNodes, int bNodes)
    {
	int gapA, gapB,i,aD,bD;
	graph=new BiGraph();
	graphPanel.setGraph(graph);
	gpSize=graphPanel.getSize();

 	gapA=(int)(((double)gpSize.width)/(aNodes+1)+0.5);
 	gapB=(int)(((double)gpSize.width)/(bNodes+1)+0.5);

	//den Durchmesser abhngig von der Panelbreite ermitteln
	aD=gpSize.width/aNodes;
	bD=gpSize.width/bNodes;
	if(aD<minVertexDiameter) return;
	if(bD<minVertexDiameter) return;
	if(aD>maxVertexDiameter) aD=maxVertexDiameter;
	if(bD>maxVertexDiameter) bD=maxVertexDiameter;

	System.out.println(aD+":"+bD);
	for(i=0;i<aNodes;i++)
	    graph.addAVertex(i*gapA+2+gapA,gpSize.height*3/4,aD/2);
	for(i=0;i<bNodes;i++)
	    graph.addBVertex(i*gapB+2+gapB,gpSize.height/4,bD/2);
	graphPanel.repaint();
    }

    /**
     * erzeugt zufllige Kanten
     * @param wkt Wahrscheinlichkeit fr eine Kante in Prozent
     */
    public void makeRandEdges(int wkt)
    {
	int i,j;
	for(i=0;i<graph.getACount();i++)
	{
	    for(j=0;j<graph.getBCount();j++)
	    {
		if(Math.random()*100<wkt) 
		{
		    graph.addEdgeByNr(i,j);
		}	
	    }
	}
	graphPanel.repaint();
    }

    /**
     * aktualisiert die Anzeige der Datenstrukturen
     * @param uStr (HTML) String der angezeigt werden soll
     */
    public void updateData(String uStr)
    {
	dataArea.setText(uStr);
    }

    /**
     * wird nach abgeschlossenem Algorithmus aufgerufen
     * @param result Endergebnis des Alorithmusses
     */
    public void matchingDone(int result)
    {
	// AUsgangsbedingungen wieder herstellen
	controlTabbedPane.setEnabledAt(0,true);
	algStartButton.setEnabled(false);
	algStopButton.setEnabled(false);
	algFFButton.setEnabled(false);
	updateGraphPanel();

	//eine Info-Box mit dem Ergebnis anzeigen
	String wMes;
	switch(result)
	{
 	   case 0:
	       wMes=new String("Kein volles Source-Match gefunden.");
	       break;
	   case 1:
	       wMes=new String("Kein Match mglich. Anzahl Sinks < Anzahl Sources.");
	       break;
	   case 2:
	       wMes=new String("Match gefunden (Keine Source-Knoten -> immer ein Match).");
	       break;
	   case 3:
	       wMes=new String("Volles Source-Match gefunden.");
	       break;
	   case 4:
	       wMes=new String("Maximales Match gefunden.");
	       break;
	   default:
	       wMes=new String(" ");
	       break;
	}
	JOptionPane.showMessageDialog(null,
				      "Algorithmus abgeschlossen. "+wMes,
				      "Info ...",
				      JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * veranlasst eine repaint des GraphPanels
     */
    public void updateGraphPanel()
    {
	graphPanel.repaint();
    }

    /**
     * setzt die Markierung einer Zeile im Algorithmus-Panel
     */
    public void setHighlight(int l)
    {
	algoPanel.setHighlight(l);
    }
    
}




